/********************************************************************
* Description: dualbipod, derived from trivkins.c
*   Kinematics for a dual bipod machine
*   1st bipod x-y plane, 2nd a-b plane
*
*   Derived from a work by Fred Proctor & Will Shackleford
*   20.05.2009: added x00, y00 to enable offsets at reference point
*   04.03.2010: changed to joint0/1 & 3/4
*
* Author:
* License: GPL Version 2
* System: Linux
*    
* Copyright (c) 2004 All rights reserved.
*
* Last change:
* $Revision: 5 $
* $Author: GerhardGleixner@web.de $
* $Date: 2010-02-21 22:39:46 +0100 (So, 21 Feb 2010) $
********************************************************************/

#include "kinematics.h"		/* these decls */
#include "rtapi_math.h"

#include "hal.h"

#include "posemath.h"
#include "rtapi.h"


struct pars {
    hal_float_t ratio, dist, L0, L1, X00, Y00;
};

struct pars * parMain = 0;

int kinematicsForward(const double *joints,

		      EmcPose * pos,
		      const KINEMATICS_FORWARD_FLAGS * fflags,
		      KINEMATICS_INVERSE_FLAGS * iflags)
{

	double j1, j2, x, y_s; 
	double Bx = parMain -> dist; 

	j1 = joints[0];
	j2 = joints[1];
	x = (j1*j1 - j2*j2 + Bx*Bx) / (2*Bx);
	y_s = j1*j1 - x*x; 
	
	if(y_s<0) return -1;
	
    pos->tran.x = x; 
    pos->tran.y = sqrt(y_s); 
	
	/*
	pos->tran.x = joints[0];
	pos->tran.y = joints[0] / (parMain -> ratio);
	*/
	
    pos->tran.z = joints[2];


	
    return 0;
}

int kinematicsInverse(const EmcPose * pos,
		      double *joints,
		      const KINEMATICS_INVERSE_FLAGS * iflags,
		      KINEMATICS_FORWARD_FLAGS * fflags)
{
	double x, y;
	double Bx =0;//= parMain -> dist; 
	
	x = pos->tran.x;
	y = pos->tran.y;

    joints[0] = sqrt(x*x + y*y); 
    joints[1] = -sqrt((Bx-x)*(Bx-x) + y*y);
	
	/*
	joints[0] = pos->tran.x;
	joints[1] = pos->tran.x * (parMain -> ratio);
	*/
	
    joints[2] = pos->tran.z;

    return 0;
}

/* implemented for these kinematics as giving joints preference */
/* when is this fct called ???? seems never */
int kinematicsHome(EmcPose * world,
		   double *joint,
		   KINEMATICS_FORWARD_FLAGS * fflags,
		   KINEMATICS_INVERSE_FLAGS * iflags)
{
    *fflags = 0;
    *iflags = 0;


    return kinematicsForward(joint, world, fflags, iflags);
}

KINEMATICS_TYPE kinematicsType()
{
    return KINEMATICS_BOTH;
}

#ifdef RTAPI
#include "rtapi.h"		/* RTAPI realtime OS API */
#include "rtapi_app.h"		/* RTAPI realtime module decls */
#include "hal.h"

#define HAL_SUCCESS 0

EXPORT_SYMBOL(kinematicsType);
EXPORT_SYMBOL(kinematicsForward);
EXPORT_SYMBOL(kinematicsInverse);
EXPORT_SYMBOL(kinematicsHome);
MODULE_LICENSE("GPL");

int comp_id;
int rtapi_app_main(void) 
{
  //struct hal_dat* par;
  int res = 0;

   comp_id = hal_init("doublespeed");
   if(comp_id < 0) return comp_id;

   parMain = hal_malloc(sizeof(*parMain));
   if(!parMain) goto error;

   if((res = hal_param_float_new("doublespeed.setratio", HAL_RW, &parMain->ratio, comp_id)) != HAL_SUCCESS) goto error;
   if((res = hal_param_float_new("doublespeed.setdist", HAL_RW, &parMain->dist, comp_id)) != HAL_SUCCESS) goto error;
   if((res = hal_param_float_new("doublespeed.setL0", HAL_RW, &parMain->L0, comp_id)) != HAL_SUCCESS) goto error;
   if((res = hal_param_float_new("doublespeed.setL1", HAL_RW, &parMain->L1, comp_id)) != HAL_SUCCESS) goto error;
   if((res = hal_param_float_new("doublespeed.setX00", HAL_RW, &parMain->X00, comp_id)) != HAL_SUCCESS) goto error;
   if((res = hal_param_float_new("doublespeed.setY00", HAL_RW, &parMain->Y00, comp_id)) != HAL_SUCCESS) goto error;
   
   
   hal_ready(comp_id);
   return 0;

error:
    hal_exit(comp_id);
    return res;
}

void rtapi_app_exit(void) { hal_exit(comp_id); }
#endif
